#include "MenuState.h"
#include "GameState.h"
#include "Globals.h"

// No logo in debug mode to bring the binary size down
#ifdef _DEBUG
const unsigned char logo[] PROGMEM = { 0x0, 0x0,};
#else
const unsigned char logo[] PROGMEM = { 0x7c, 0x18, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xfe, 0x00, 0xff, 0xff, 0xff, 0xff, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x1f, 0x1f, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x1f, 0x1f, 0x00, 0xff, 0xff, 0xff, 0xff, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x1f, 0x1f, 0x00, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x1f, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0x1f, 0x00, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xfe, 0x00, 0x00, 0xfe, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff, 0xff, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xc0, 0xc0, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xf, 0xf, 0xf, 0xf, 0xf, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xc0, 0xc0, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc0, 0xff, 0xff, 0xff, 0xff, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x3, 0x00, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x00, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x00, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x3, 0x00, 0x00, 0x3, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x00, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x00, 0x7, 0x7, 0x7, 0x7, 0x00, 0x00, 0x3, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x7, 0x3, };
#endif

const uint16_t beep[] PROGMEM = { 255,100,TONES_END };

#ifdef _DEBUG
const char scrollText[] PROGMEM = "DEBUG";

#else
const char scrollText[] PROGMEM = " ... VERSION 1.2 ... BY ALEX MCCHESNEY ... WASHINGUP.ITCH.IO ... ";
#endif

MenuState::MenuState()
{
}

void MenuState::update() 
{
	if (arduboy.justPressed(UP_BUTTON) || arduboy.justPressed(DOWN_BUTTON))
	{
		startSelected = !startSelected;
		sound.tones(beep);
	}

	if (arduboy.justPressed(A_BUTTON) || arduboy.justPressed(B_BUTTON))
	{
		if(startSelected)
		{ 
			setState(STATE_GAME);
		}
		else
		{
			arduboy.audio.toggle();
			arduboy.audio.saveOnOff();
		}
	}

	renderSprite(logo, Vector2Int(2, 0));

	drawMenu(49, 30, startSelected, F("START"));

	if (arduboy.audio.enabled())
	{
		drawMenu(40, 42, !startSelected, F("SOUND:ON"));
	}
	else
	{
		drawMenu(37, 42, !startSelected, F("SOUND:OFF"));
	}

	drawScrollText();
}

void MenuState::drawMenu(uint8_t x, uint8_t y, bool selected, const __FlashStringHelper* text)
{
	if (selected)
	{
		x -= 6;
		arduboy.setCursor(x, y);
		arduboy.print(F(">"));
	}
	else
	{
		arduboy.setCursor(x, y);
	}
	
	arduboy.print(text);

	if (selected)
	{
		arduboy.print(F("<"));
	}
}

void MenuState::drawScrollText()
{
	arduboy.setCursor(scrollTextPos, 56);
	arduboy.print(F("HIGH SCORE: "));
	arduboy.print(getHighScore());
	arduboy.print((__FlashStringHelper*)scrollText);

	scrollTextPos--;
	if (scrollTextPos < -512)
	{
		scrollTextPos = 128;
	}


}